"use strict";
const path = require('path');
const fs = require('fs');
const child_process_1 = require('child_process');
const errors_1 = require('./errors');
class GitProcess {
    /**
     *  Find the path to the embedded Git environment
     */
    static resolveGitDir() {
        if (process.env.TEST) {
            return path.join(__dirname, '..', 'git');
        }
        else {
            return path.join(__dirname, '..', '..', 'git');
        }
    }
    /**
     *  Find the path to the embedded Git binary
     */
    static resolveGitBinary() {
        const gitDir = GitProcess.resolveGitDir();
        if (process.platform === 'darwin') {
            return path.join(gitDir, 'bin', 'git');
        }
        else if (process.platform === 'win32') {
            return path.join(gitDir, 'cmd', 'git.exe');
        }
        throw new Error('Git not supported on platform: ' + process.platform);
    }
    /** Find the path to the embedded git exec path. */
    static resolveGitExecPath() {
        const gitDir = GitProcess.resolveGitDir();
        if (process.platform === 'darwin') {
            return path.join(gitDir, 'libexec', 'git-core');
        }
        else if (process.platform === 'win32') {
            return path.join(gitDir, 'mingw64', 'libexec', 'git-core');
        }
        throw new Error('Git not supported on platform: ' + process.platform);
    }
    static pathExists(path) {
        try {
            fs.accessSync(path, fs.F_OK);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    /**
     * Execute a command and read the output using the embedded Git environment.
     *
     * The returned promise will only reject when the git executable failed to launch.
     * See the result's `stderr` and `exitCode` for any potential error information.
     */
    static exec(args, path, options) {
        return new Promise(function (resolve, reject) {
            const gitLocation = GitProcess.resolveGitBinary();
            let envPath = process.env.PATH || '';
            if (process.platform === 'win32') {
                const gitDir = GitProcess.resolveGitDir();
                envPath = `${gitDir}\\mingw64\\bin;${envPath}`;
            }
            const env = Object.assign({}, process.env, {
                GIT_EXEC_PATH: GitProcess.resolveGitExecPath(),
                PATH: envPath,
            }, options ? options.env : {});
            if (process.platform === 'win32') {
                // while reading the environment variable is case-insensitive
                // you can create a hash with multiple values, which means the
                // wrong value might be used when spawning the child process
                //
                // this ensures we only ever supply one value for PATH
                if (env.Path) {
                    delete env.Path;
                }
            }
            // Explicitly annotate opts since typescript is unable to infer the correct
            // signature for execFile when options is passed as an opaque hash. The type
            // definition for execFile currently infers based on the encoding parameter
            // which could change between declaration time and being passed to execFile.
            // See https://git.io/vixyQ
            const execOptions = {
                cwd: path,
                encoding: 'utf8',
                maxBuffer: 10 * 1024 * 1024,
                env
            };
            const spawnedProcess = child_process_1.execFile(gitLocation, args, execOptions, function (err, stdout, stderr) {
                const code = err ? err.code : 0;
                if (code === errors_1.NotFoundExitCode) {
                    if (GitProcess.pathExists(path) === false) {
                        reject(new Error('Unable to find path to repository on disk.'));
                        return;
                    }
                    reject(new Error('Git could not be found. This is most likely a problem in git-kitchen-sink itself.'));
                    return;
                }
                resolve({ stdout, stderr, exitCode: code });
            });
            if (options && options.stdin) {
                // See https://github.com/nodejs/node/blob/7b5ffa46fe4d2868c1662694da06eb55ec744bde/test/parallel/test-stdin-pipe-large.js
                spawnedProcess.stdin.end(options.stdin, options.stdinEncoding);
            }
            if (options && options.processCallback) {
                options.processCallback(spawnedProcess);
            }
        });
    }
    /** Try to parse an error type from stderr. */
    static parseError(stderr) {
        for (const regex in errors_1.GitErrorRegexes) {
            if (stderr.match(regex)) {
                const error = errors_1.GitErrorRegexes[regex];
                return error;
            }
        }
        return null;
    }
}
exports.GitProcess = GitProcess;
//# sourceMappingURL=git-process.js.map